# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import pytest
from invenio_app.factory import create_api as _create_api


@pytest.fixture(scope="module")
def extra_entry_points():
    """Extra entry points to load the mock_module features."""
    return {
        'invenio_db.model': [
            'mock_module = mock_module.models',
        ],
        'invenio_jsonschemas.schemas': [
            'mock_module = mock_module.jsonschemas',
        ],
        'invenio_search.mappings': [
            'draftsresources = mock_module.mappings',
        ]
    }


@pytest.fixture(scope="module")
def create_app(instance_path, entry_points):
    """Application factory fixture."""
    return _create_api


@pytest.fixture()
def input_data():
    """Input data (as coming from the view layer)."""
    return {
        'metadata': {
            'title': 'Test'
        },
    }



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""Example resource."""

from marshmallow import Schema
from marshmallow_utils.fields import Link
from uritemplate import URITemplate

from invenio_drafts_resources.resources import \
    DraftActionResource as DraftActionResourceBase
from invenio_drafts_resources.resources import \
    DraftActionResourceConfig as DraftActionResourceConfigBase
from invenio_drafts_resources.resources import \
    DraftFileActionResource as DraftFileActionResourceBase
from invenio_drafts_resources.resources import \
    DraftFileActionResourceConfig as DraftFileActionResourceConfigBase
from invenio_drafts_resources.resources import \
    DraftFileResource as DraftFileResourceBase
from invenio_drafts_resources.resources import \
    DraftFileResourceConfig as DraftFileResourceConfigBase
from invenio_drafts_resources.resources import \
    DraftResource as DraftResourceBase
from invenio_drafts_resources.resources import \
    DraftResourceConfig as DraftResourceConfigBase
from invenio_drafts_resources.resources import \
    DraftVersionResource as DraftVersionResourceBase
from invenio_drafts_resources.resources import \
    DraftVersionResourceConfig as DraftVersionResourceConfigBase
from invenio_drafts_resources.resources import \
    RecordFileActionResource as RecordFileActionResourceBase
from invenio_drafts_resources.resources import \
    RecordFileActionResourceConfig as RecordFileActionResourceConfigBase
from invenio_drafts_resources.resources import \
    RecordFileResource as RecordRecordFileResourceBase
from invenio_drafts_resources.resources import \
    RecordFileResourceConfig as RecordFileResourceConfigBase
from invenio_drafts_resources.resources import \
    RecordResource as RecordResourceBase
from invenio_drafts_resources.resources import \
    RecordResourceConfig as RecordResourceConfigBase


class RecordLinksSchema(Schema):
    """Schema for a record's links."""

    self = Link(
        template=URITemplate("/api/mocks/{pid_value}"),
        permission="read",
        params=lambda record: {'pid_value': record.pid.pid_value}
    )
    # TODO: Add delete, files, ...


class DraftLinksSchema(Schema):
    """Schema for a draft's links."""

    self = Link(
        template=URITemplate("/api/mocks/{pid_value}/draft"),
        permission="read",
        params=lambda draft: {'pid_value': draft.pid.pid_value}
    )
    publish = Link(
        template=URITemplate("/api/mocks/{pid_value}/draft/actions/publish"),
        permission="publish",
        params=lambda draft: {'pid_value': draft.pid.pid_value}
    )


##
# RECORDS
##
class RecordResourceConfig(RecordResourceConfigBase):
    """Mock record resource configuration."""

    list_route = "/mocks"
    item_route = f"{list_route}/<pid_value>"

    links_config = {
        "record": RecordLinksSchema
    }

    # NOTE: Developers using drafts-resources need to do this
    draft_links_config = {
        "record": DraftLinksSchema
    }


class RecordResource(RecordResourceBase):
    """Mock record resource."""

    default_config = RecordResourceConfig


class RecordFileResourceConfig(RecordFileResourceConfigBase):
    """Mock record file resource."""

    item_route = "/mocks/<pid_value>/files/<key>"
    list_route = "/mocks/<pid_value>/files"


class RecordFileResource(RecordRecordFileResourceBase):
    """Mock record file resource config."""

    default_config = RecordFileResourceConfig


class RecordFileActionResourceConfig(RecordFileActionResourceConfigBase):
    """Mock record file resource."""

    list_route = "/mocks/<pid_value>/files/<key>/<action>"


class RecordFileActionResource(RecordFileActionResourceBase):
    """Mock record file resource config."""

    default_config = RecordFileActionResourceConfig


##
# DRAFTS
##
class DraftResourceConfig(DraftResourceConfigBase):
    """Mock draft resource configuration."""

    list_route = "/mocks/<pid_value>/draft"

    links_config = {
        # TODO: Revisit naming for "record"?
        "record": DraftLinksSchema
    }


class DraftResource(DraftResourceBase):
    """Mock draft resource."""

    default_config = DraftResourceConfig


class DraftActionResourceConfig(DraftActionResourceConfigBase):
    """Mock draft action resource configuration."""

    list_route = "/mocks/<pid_value>/draft/actions/<action>"

    action_commands = {
        "create": {
            "publish": "publish",
            "command": "not_implemented"
        }
    }

    links_config = {
        "record": DraftLinksSchema
    }

    record_links_config = {
        "record": RecordLinksSchema
    }


class DraftActionResource(DraftActionResourceBase):
    """Mock draft action resource."""

    default_config = DraftActionResourceConfig

    def create_command(self, action, operation):
        """Dummy handler."""
        return self._get_cmd_func(action, operation).to_dict(), 200


class DraftFileResourceConfig(DraftFileResourceConfigBase):
    """Mock record file resource."""

    item_route = "/mocks/<pid_value>/draft/files/<key>"
    list_route = "/mocks/<pid_value>/draft/files"


class DraftFileResource(DraftFileResourceBase):
    """Mock record file resource config."""

    default_config = DraftFileResourceConfig


class DraftFileActionResourceConfig(DraftFileActionResourceConfigBase):
    """Mock record file resource."""

    list_route = "/mocks/<pid_value>/draft/files/<key>/<action>"


class DraftFileActionResource(DraftFileActionResourceBase):
    """Mock record file resource config."""

    default_config = DraftFileActionResourceConfig


##
# VERSIONING
##
class DraftVersionResourceConfig(DraftVersionResourceConfigBase):
    """Mock draft version resource configuration."""

    list_route = "/mocks/<pid_value>/versions"


class DraftVersionResource(DraftVersionResourceBase):
    """Mock draft version resource."""

    default_config = DraftVersionResourceConfig



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""
from uuid import uuid4

import pytest
from flask_principal import Identity, Need, UserNeed
from mock_module.api import Draft
from mock_module.service import Service


@pytest.fixture()
def identity_simple():
    """Simple identity fixture."""
    i = Identity(1)
    i.provides.add(UserNeed(1))
    i.provides.add(Need(method='system_role', value='any_user'))
    return i


@pytest.fixture()
def service(appctx):
    """Service instance."""
    return Service()


@pytest.fixture()
def example_record(app, db):
    """Example record."""
    record = Draft.create({}, metadata={'title': 'Test'})
    db.session.commit()
    return record



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import pytest
from flask_principal import Identity, Need, UserNeed
from mock_module.resource import DraftActionResource, \
    DraftActionResourceConfig, DraftFileActionResource, \
    DraftFileActionResourceConfig, DraftFileResource, \
    DraftFileResourceConfig, DraftResource, DraftResourceConfig, \
    DraftVersionResource, DraftVersionResourceConfig, \
    RecordFileActionResource, RecordFileActionResourceConfig, \
    RecordFileResource, RecordFileResourceConfig, RecordResource, \
    RecordResourceConfig
from mock_module.service import Service, ServiceConfig


@pytest.fixture(scope="module")
def record_resource():
    """Record resource."""
    # FIXME
    # This should work but doesn't because the application context is checked
    # to see if it's been overridden in the config.
    # return Resource(service=Service())
    return RecordResource(
        config=RecordResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def record_file_resource():
    """Record file resource."""
    return RecordFileResource(
        config=RecordFileResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def record_file_action_resource():
    """Record file action resource."""
    return RecordFileActionResource(
        config=RecordFileActionResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def draft_resource():
    """Draft resource."""
    # FIXME
    # This should work but doesn't because the application context is checked
    # to see if it's been overridden in the config.
    # return Resource(service=Service())
    return DraftResource(
        config=DraftResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def draft_action_resource():
    """Draft action resource."""
    # FIXME
    # This should work but doesn't because the application context is checked
    # to see if it's been overridden in the config.
    # return Resource(service=Service())
    return DraftActionResource(
        config=DraftActionResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def version_resource():
    """Draft version Resource."""
    # FIXME
    # This should work but doesn't because the application context is checked
    # to see if it's been overridden in the config.
    # return Resource(service=Service())
    return DraftVersionResource(
        config=DraftVersionResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def draft_file_resource():
    """Draft file resource."""
    return DraftFileResource(
        config=DraftFileResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def draft_file_action_resource():
    """Draft file action resource."""
    return DraftFileActionResource(
        config=DraftFileActionResourceConfig,
        service=Service(config=ServiceConfig)
    )


@pytest.fixture(scope="module")
def base_app(
        base_app, record_resource, record_file_resource,
        record_file_action_resource, draft_resource, draft_action_resource,
        version_resource, draft_file_resource, draft_file_action_resource):
    """Application factory fixture."""
    # records
    base_app.register_blueprint(record_resource.as_blueprint('mock_record'))
    # record files
    base_app.register_blueprint(
        record_file_resource.as_blueprint('mock_record_files'))
    base_app.register_blueprint(
        record_file_action_resource.as_blueprint('mock_record_files_action'))
    # drafts
    base_app.register_blueprint(draft_resource.as_blueprint('mock_draft'))
    base_app.register_blueprint(
        draft_action_resource.as_blueprint('mock_draft_action'))
    base_app.register_blueprint(version_resource.as_blueprint('mock_version'))
    # draft files
    base_app.register_blueprint(
        draft_file_resource.as_blueprint('mock_draft_files'))
    base_app.register_blueprint(
        draft_file_action_resource.as_blueprint('mock_draft_files_action'))
    # FIXME: Why is this commented out? Also in records-resources
    # base_app.register_error_handler(HTTPException, handle_http_exception)
    yield base_app


@pytest.fixture(scope="module")
def identity_simple():
    """Simple identity fixture."""
    i = Identity(1)
    i.provides.add(UserNeed(1))
    i.provides.add(Need(method='system_role', value='any_user'))
    return i


@pytest.fixture(scope="module")
def headers():
    """Default headers for making requests."""
    return {
        'content-type': 'application/json',
        'accept': 'application/json',
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""RecordDraft File Service API."""


from invenio_records_resources.services import RecordFileService

from .config import FileMetadataServiceConfig, FileServiceConfig


class FileService(RecordFileService):
    """RecordDraft File Service."""

    # NOTE: We can just rely on Records-Resources FileService.
    # Only config is needed.
    default_config = FileServiceConfig


class FileMetadataService(RecordFileService):
    """RecordDraft File Metadata Service."""

    # NOTE: We can just rely on Records-Resources FileService.
    # Only config is needed.
    default_config = FileMetadataServiceConfig



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Resources module to create REST APIs."""

from .config import DraftFileActionResourceConfig, DraftFileResourceConfig, \
    RecordFileActionResourceConfig, RecordFileResourceConfig
from .resource import DraftFileActionResource, DraftFileResource, \
    RecordFileActionResource, RecordFileResource

__all__ = (
    "DraftFileResourceConfig",
    "DraftFileResource",
    "DraftFileActionResourceConfig",
    "DraftFileActionResource",
    "RecordFileResourceConfig",
    "RecordFileActionResourceConfig",
    "RecordFileResource",
    "RecordFileActionResource",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Test files resource integration."""

from io import BytesIO

import pytest


@pytest.mark.skip()
def test_files_publish_flow(client, es_clear, location, headers):
    """Test record creation."""
    h = headers
    # Create a draft
    res = client.post("/mocks", headers=h, json={
        'metadata': {'title': 'Test'}
    })
    assert res.status_code == 201
    id_ = res.json['id']

    # Initialize files upload
    res = client.post(f"/mocks/{id_}/draft/files", headers=h, json=[
        {'key': 'test.pdf'}
    ])
    assert res.status_code == 201
    assert res.json['entries'][0]['key'] == 'test.pdf'
    assert res.json['entries'][0]['status'] == 'pending'

    # Upload a file
    res = client.put(
        f"/mocks/{id_}/draft/files/test.pdf/content",
        headers={'content-type': 'application/octet-stream'},
        data=BytesIO(b'testfile'),
    )
    assert res.status_code == 200

    # Commit the file
    res = client.post(f"/mocks/{id_}/draft/files/test.pdf/commit", headers=h)
    assert res.status_code == 200
    assert res.json['key'] == 'test.pdf'
    assert res.json['status'] == 'completed'

    # Publish the record
    res = client.post(f"/mocks/{id_}/draft/actions/publish", headers=h)
    assert res.status_code == 202

    # Check published files
    res = client.get(f"/mocks/{id_}/files", headers=h)
    assert res.status_code == 200
    assert res.json['key'] == 'test.pdf'
    assert res.json['status'] == 'completed'

    # Edit the record
    res = client.post(f"/mocks/{id_}/draft", headers=h)
    assert res.status_code == 201

    # Publish again
    res = client.post(f"/mocks/{id_}/draft/actions/publish", headers=h)
    assert res.status_code == 202

    # Check published files
    res = client.get(f"/mocks/{id_}/files", headers=h)
    assert res.status_code == 200
    assert res.json['key'] == 'test.pdf'
    assert res.json['status'] == 'completed'


@pytest.mark.skip()
def test_metadata_only_record(client, es_clear, location, headers):
    """Test record with files disabled."""
    h = headers
    # Create a draft
    res = client.post("/mocks", headers=h, json={
        'metadata': {'title': 'Test'}
    })
    assert res.status_code == 201
    id_ = res.json['id']

    # Disable files
    res = client.put(f"/mocks/{id_}/draft/files", headers=h, json={
        'enabled': False,
    })
    assert res.status_code == 200
    assert res.json['enabled'] is False
    assert 'entries' not in res.json

    # Publish the record
    res = client.post(f"/mocks/{id_}/draft/actions/publish", headers=h)
    assert res.status_code == 202

    # Check published files
    res = client.get(f"/mocks/{id_}/files", headers=h)
    assert res.status_code == 200
    assert res.json['enabled'] is False
    assert 'entries' not in res.json

    # Edit the record
    res = client.post(f"/mocks/{id_}/draft", headers=h)
    assert res.status_code == 201

    # Publish again
    res = client.post(f"/mocks/{id_}/draft/actions/publish", headers=h)
    assert res.status_code == 202

    # Check published files
    res = client.get(f"/mocks/{id_}/files", headers=h)
    assert res.status_code == 200
    assert res.json['enabled'] is False
    assert 'entries' not in res.json



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""RecordDraft File Service API config."""

from invenio_records_resources.services import \
    FileServiceConfig as FileServiceConfigBase


class FileServiceConfig(FileServiceConfigBase):
    """Draft File Service configuration."""

    pass


class FileMetadataServiceConfig(FileServiceConfigBase):
    """Draft File Metadata Service configuration."""

    # TODO: FILL ME!
    pass



"""Example of a record model."""

from invenio_db import db
from invenio_records.models import RecordMetadataBase

from invenio_drafts_resources.records import DraftMetadataBase


class DraftMetadata(db.Model, DraftMetadataBase):
    """Model for mock module metadata."""

    __tablename__ = 'draft_mock_metadata'


class RecordMetadata(db.Model, RecordMetadataBase):
    """Model for mock module metadata."""

    __tablename__ = 'record_mock_metadata'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Module tests."""

from datetime import datetime

import pytest
from invenio_indexer.api import RecordIndexer
from mock_module.api import Draft


@pytest.fixture()
def example_record(db, input_data):
    """Example record."""
    record = Draft.create(
        input_data,
        expires_at=datetime(2020, 9, 7, 0, 0)
    )
    db.session.commit()
    return record


@pytest.fixture()
def indexer():
    """Indexer instance with correct Record class."""
    return RecordIndexer(
        record_cls=Draft, record_to_index=lambda r: (r.index._name, '_doc')
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""File resource configuration."""

from flask_resources.resources import ResourceConfig
from invenio_records_resources.resources import FileActionResourceConfig
from invenio_records_resources.resources import \
    FileResourceConfig as RecordFileResourceConfig
from invenio_records_resources.resources.actions import ActionResourceConfig


class RecordFileActionResourceConfig(FileActionResourceConfig):
    """Record resource config."""

    list_route = "/records/<pid_value>/files/<key>/<action>"
    action_commands = {
        'create': {},
        'read': {
            'content': 'download_file'
        },
        'update': {},
        'delete': {}
    }


class DraftFileResourceConfig(RecordFileResourceConfig):
    """Record resource config."""

    item_route = "/records/<pid_value>/draft/files/<key>"
    list_route = "/records/<pid_value>/draft/files"


class DraftFileActionResourceConfig(FileActionResourceConfig):
    """Record resource config."""

    list_route = "/records/<pid_value>/draft/files/<key>/<action>"
    action_commands = {
        'create': {
            'commit': 'commit_file'
        },
        'read': {
            'content': 'get_file_content'
        },
        'update': {
            'content': 'set_file_content'
        },
        'delete': {}
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Data access layer tests."""

import pytest
from invenio_search import current_search_client
from jsonschema import ValidationError
from mock_module.api import Draft


#
# Smoke tests
#
def test_record_empty(app, db):
    """Test record creation."""
    # Empty record creation works, and injects a schema.
    record = Draft.create({})
    db.session.commit()
    assert record.schema

    # JSONSchema validation works.
    pytest.raises(
        ValidationError,
        Draft.create,
        {'metadata': {'title': 1}}
    )


def test_record_via_field(app, db):
    """Record creation via field."""
    record = Draft.create({}, metadata={'title': 'test'})
    assert record.metadata == {'title': 'test'}


def test_record_indexing(app, db, es, example_record, indexer):
    """Test indexing of a record."""
    # Index document in ES
    assert indexer.index(example_record)['result'] == 'created'

    # Retrieve document from ES
    data = current_search_client.get(
        'draftsresources-drafts-draft-v1.0.0',
        id=example_record.id,
        doc_type='_doc'
    )

    # Loads the ES data and compare
    record = Draft.loads(data['_source'])
    assert record == example_record
    assert record.id == example_record.id
    assert record.revision_id == example_record.revision_id
    assert record.created == example_record.created
    assert record.updated == example_record.updated
    assert record.expires_at == example_record.expires_at

    # Check system fields
    record.metadata == example_record['metadata']


def test_record_delete_reindex(app, db, es, example_record, input_data,
                               indexer):
    """Test reindexing of a deleted record."""
    record = example_record

    # Index record
    assert indexer.index(record)['result'] == 'created'

    # Delete record.
    record.delete()
    db.session.commit()
    assert indexer.delete(record)['result'] == 'deleted'

    # Update record and reindex (this will cause troubles unless proper
    # optimistic concurrency control is used).
    record.undelete()
    record.commit()
    db.session.commit()
    assert indexer.index(record)['result'] == 'created'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs."""

from flask import g
from flask_resources.context import resource_requestctx
from invenio_records_resources.resources import \
    RecordResource as _RecordResource

from .config import RecordResourceConfig


class RecordResource(_RecordResource):
    """Draft-aware RecordResource."""

    default_config = RecordResourceConfig

    def create(self):
        """Create an item."""
        data = resource_requestctx.request_content
        item = self.service.create(
            g.identity, data, links_config=self.config.draft_links_config)
        return item.to_dict(), 201

    def search(self):
        """Perform a search over the items."""
        identity = g.identity
        status = resource_requestctx.url_args.pop("status", None)

        if not status:  # To account for `?status=`, which ends up in []
            status = ["published"]

        hits = self.service.search(
            identity=identity,
            params=resource_requestctx.url_args,
            links_config=self.config.links_config,
            status="published" if "published" in status else "draft",
        )
        return hits.to_dict(), 200



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""RecordDraft Services."""

from .config import RecordDraftServiceConfig
from .service import RecordDraftService

__all__ = (
    'RecordDraftService',
    'RecordDraftServiceConfig',
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs."""

from invenio_records_resources.resources import RecordResourceConfig
from invenio_records_resources.resources.actions import ActionResourceConfig

from ..records import RecordLinksSchema
from .schemas_links import DraftLinksSchema


class DraftResourceConfig(RecordResourceConfig):
    """Draft resource config."""

    list_route = "/records/<pid_value>/draft"
    item_route = None

    links_config = {
        "record": DraftLinksSchema
    }


class DraftActionResourceConfig(ActionResourceConfig):
    """Draft action resource config."""

    list_route = "/records/<pid_value>/draft/actions/<action>"
    item_route = None  # To avoid issues, due to inheritance.

    action_commands = {
        "create": {
            "publish": "publish",
        }
    }

    record_links_config = {
        "record": RecordLinksSchema
    }


class DraftVersionResourceConfig(RecordResourceConfig):
    """Draft resource config."""

    list_route = "/records/<pid_value>/versions"
    # TODO: REVISIT



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Records service copmonent base classes."""

from invenio_records_resources.services.records.components import \
    FilesComponent, MetadataComponent, ServiceComponent


class DraftFilesComponent(FilesComponent):
    """Draft files service component."""

    # TODO: Add tests for publishing a draft with files
    def publish(self, draft=None, record=None):
        """Copy bucket and files to record."""
        draft_files = draft.files
        bucket = draft_files.bucket
        if draft.files.enabled and bucket:
            # Set the draft bucket on the record also
            record.bucket = bucket
            record.bucket_id = bucket.id

            # Lock the bucket
            # TODO: Fix issues in REST API and UI for not allowing files to be
            # modified after editing a published record.
            # bucket.locked = True

            # TODO: actually "sync"
            # Copy over the files
            for key, df in draft_files.items():
                # TODO: Fix __setitem__ in FilesField, to support `None` for
                # metadata
                if df.metadata is not None:
                    record.files[key] = df.object_version, df.metadata
                else:
                    record.files[key] = df.object_version
        # TODO: Normally we don't need this... on record creation from a draft
        # with disabled files, the record should start with `enabled=False`.
        elif not draft.files.enabled:
            record.files.enabled = False


class RelationsComponent(ServiceComponent):
    """Service component for PID relations integration."""

    # PIDNodeVersioning(pid=conceptrecid).insert_draft_child(child_pid=recid)


class DraftMetadataComponent(MetadataComponent):
    """Service component for draft metadata integration."""

    def update_draft(self, *args, **kwargs):
        """Update draft metadata."""
        self.update(*args, **kwargs)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Record File Resources."""

# from flask import abort, g
from flask_resources import CollectionResource
from invenio_records_resources.config import ConfigLoaderMixin
from invenio_records_resources.resources import FileActionResource, \
    FileResource
from invenio_records_resources.resources.actions import ActionResource

from .config import DraftFileActionResourceConfig, DraftFileResourceConfig, \
    RecordFileActionResourceConfig, RecordFileResourceConfig


class RecordFileResource(CollectionResource, ConfigLoaderMixin):
    """Record file resource.

    It does not inherit from Invenio-Records-Resources::FileResource to
    disable the queries to any other endpoint (e.g. create), which otherwise
    would need to be overwritten. Here they are merely not implemented.
    """

    default_config = RecordFileResourceConfig

    def __init__(self, config=None, service=None):
        """Constructor."""
        super(RecordFileResource, self).__init__(
            config=self.load_config(config))
        self.service = None  # Force user's set up

    # ListView GET
    def search(self, *args, **kwargs):
        """List files."""
        return None, 200

    # ItemView GET
    def read(self, *args, **kwargs):
        """Read a single file."""
        return None, 200


# ActionResource inherits ConfigLoaderMixin
class RecordFileActionResource(ActionResource):
    """Record file action resource."""

    default_config = RecordFileActionResourceConfig


class DraftFileResource(FileResource):
    """File resource."""

    default_config = DraftFileResourceConfig

    # TODO: No constructor for now.
    # Required when service level implementation.


# ActionResource inherits ConfigLoaderMixin
class DraftFileActionResource(FileActionResource):
    """File action resource.

    NOTE: `Commit` exists as an action to avoid having to split the
    `FileResource` into Collection + Singleton to have to POST operations.
    """

    default_config = DraftFileActionResourceConfig
